		TITLE	Halls Of The Things

		; This file ( S.Brattel )

		include SYMBOLS.INC	; General equates

		include HSYM.INC	; Don't show them all here

CODE		segment word public	; Usual gumph 
		assume cs:CODE		; ds and es are used variously

		; Start here

Start:		mov ax,StackSeg		; Setup stack
		mov SS,ax		;
		mov SP,offset TOS	;

		; Now we start the game . . .

		call Init_VDU		; Set correct modes, etc.

		call Set_VDU_Front	; Set to front chars

		; Now drive memory

		call Deallocate		; Release unused RAM
		mov di,o sError1	;
		jc Error		; Can't, machine fault

		call Get_BigMaze	; Get enough RAM for Big maze (300k)
		mov di,o sError2	;
		jc Error		; Can't, not enough RAM

		call Get_Monsters	; Get enough RAM for monster table
		mov di,o sError2	;
		jc Error		; Can't, not enough RAM

		; Play the game . . . 

		call Play_Game		; Play the game

		; Exit routine

Exit_Game:	call Close_VDU		; Done

		mov ah,Terminate	; Exit
		int dos			;

; Simple error handler

Error:		mov	dx,00600h	; Message start coordinates
		call	Print_Str_Code	;
		jmp	Exit_Game	;

; Now run a small demo

Play_Game:	call Set_VDU_Game	; Set to game chars

		call Link_Kbd		; Get control of Kbd

		call Show_Chars		; Show the char set

		; Pause

;		call Pause		; Wait for escape

		; Setup the viewport

		mov w cs:ScreenX,-1	; Top-left
		mov w cs:ScreenY, 0	;

		; Now draw the maze

		call init_small_maze	;
		call build_small_maze	;
		call draw_small_maze	;
		call expand		;

		call init_objects	;
		call init_things	;
		call init_proj		;

		; Setup data

		call Init_Status	; Setup the score, bargraphs etc.

		; Show the maze

		call ReShow_Window	; Show maze

		; Neither man has moved

		mov b cs:Man_Look,0	; Neither

		; Game Loop

ShowLp:		call Rand_32		; Scramble

		; Get the man keys

		call Get_Man_Keys	; Get the mans' keys

		; Has man 0 moved ?

		test b cs:Man_Look,01h	; Has man 0 moved ?
		jz S_N0			; No, skip

		mov si,cs:Monster_Base	; Giz seg
		mov ds,si		;

		mov si,0		; Point at man 0		
		call Look		; Perform search if man move

		; Man 1 moved ?

S_N0:		test b cs:Man_Look,02h	; Has man 1 moved ?
		jz S_N1			; No, skip

		mov si,cs:Monster_Base	; Giz seg
		mov ds,si		;

		mov si,lMon		; Point at man 1		
		call Look		; Perform search if man move

S_N1:		call move_monsters	; Move the monsters

		call Move_Proj		; Move the projectiles

		call Show_Window	; Show maze

		call Change_Speed	; Change the game speed

		; Now, we need to wait a while. . .

		mov ax,cs:Speed		; Slow things down a bit. . .

S_D_Lp:		mov bx,10		; Slow them down a lot. . .

S_D_Lp2:	dec bx			;
		jnz S_D_Lp2		;

		dec ax			;
		jnz S_D_Lp		;

		mov al,CS:Key_Matrix+kEsc; Test for escape
		test al,001H		; End flag
		jz ShowLp		;

		call Unlink_Kbd		; Done

End_demo:	ret			; Done

; Change the speed

Change_Speed:	test b CS:Key_Matrix+kQ,1; Test for "Q"
		jnz CS_Up 		; Yes

		test b CS:Key_Matrix+kA,1; Test for "A"
		jnz CS_Down		; Yes

		ret			;

CS_Up:		mov ax,-100		;
		add cs:Speed,ax		;

		ret			;

CS_Down:	mov ax,100		;
		add cs:Speed,ax		;

		ret			;


; Setup the status display

Init_Status:	mov di,o sStatus	; Setup the bottom line

		mov dx,0000h		; Start posn

		call OutTextStr		; Write the screen

		; Init values

		mov w cs:Soul_Cnt,0	; Start values

		mov b cs:Magic_Bar,0	;

		mov b cs:Wounds_Bar,0	;

		mov b cs:Rings_Shown,255;

		ret			; Done

; Show any change to the status line

Drive_Status:	mov ah,mRingsHeld[si]	; Get the rings held
		cmp ah,cs:Rings_Shown	; No. shown
		je DS_0			; Same, skip

		; Show another ring/key

		mov cs:Rings_Shown,ah	; Store it

		mov cl,cRings		; Character

		mov dx,1806h		; xy

DSR_Lp:		mov al,0A0h		; Not held char

		shr ah,1		; Get first
		jnc DSR_1		; Not held

		mov al,cl		; Get char

DSR_1:		call OutIt		; Show it (graphic)

		inc dl			; Step X

		inc cl			; Step Ring

		cmp cl,cKey+1		; Done ?
		jne DSR_Lp		; Loop

		; Rings shown				

DS_0:		mov ax,mSouls[si]	; Get count
		cmp ax,cs:Soul_Cnt	; Same ?
		je DS_1			; Yes, skip

		; Save the score and display it

		mov cs:Soul_Cnt,ax	; Set it

		mov dx,0005h		; xy
		call Show_No		; Show it

DS_1:		mov al,mMagic[si]	; Giz magic level
		mov ah,0		; 16-bit

		add ax,2		; Convert to bar-graph levels
		shr ax,1		;
		shr ax,1		;

		mov ah,al		; Save it

		mov al,cs:Magic_Bar	; Get current
		cmp al,ah		; Same ?
		je DS_2			; Yes, skip

		; Step in the right dirn.

		mov ah,-1		; Down

		ja DS_1a		; Yes, step it

		mov ah,1		; Up

DS_1a:		add al,ah		; Step it
		mov cs:Magic_Bar,al	;

		mov dx,0024h		; xy

		call Draw_Bar		; Show it

DS_2:		mov al,mWounds[si]	; Giz wounds level
		mov ah,0		; 16-bit

		add ax,2		; Convert to bar-graph levels
		shr ax,1		;
		shr ax,1		;

		mov ah,al		; Save it

		mov al,cs:Wounds_Bar	; Get current
		cmp al,ah		; Same ?
		je DS_X			; Yes, skip

		; Step in the right dirn.

		mov ah,-1		; Down

		ja DS_2a		; Yes, step it

		mov ah,1		; Up

DS_2a:		add al,ah		; Step it
		mov cs:Wounds_Bar,al	;

		mov dx,003Fh		; xy

		call Draw_Bar		; Show it

DS_X:		ret			; Done

; Scan the man's keys and set some flags

Get_Man_Keys:	mov ah,0		; None yet

		test b CS:Key_Matrix+kZ,1; Test for left
		jz GMK_NL		; No

		or ah,01h		; Set left

GMK_NL:		test b CS:Key_Matrix+kX,1; Test for right
		jz GMK_NR		; No

		or ah,02h		; Set right

GMK_NR:		test b CS:Key_Matrix+kN,1; Test for up
		jz GMK_NU		; No

		or ah,04h		; Set up

GMK_NU:		test b CS:Key_Matrix+kM,1; Test for down
		jz GMK_ND		; No

		or ah,08h		; Set down

GMK_ND:		test b CS:Key_Matrix+kL,1; Test for lightning
		jz GMK_NLi		; No

		or ah,10h		; Set light

GMK_NLi:	test b CS:Key_Matrix+kF,1; Test for fire
		jz GMK_NF		; No

		or ah,20h		; Set fire

GMK_NF:		test b CS:Key_Matrix+kK,1; Test for keep
		jz GMK_NK		; No

		or ah,40h		; Set keep

GMK_NK:		test b CS:Key_Matrix+kD,1; Test for drop
		jz GMK_NDr		; No

		or ah,80h		; Set drop

GMK_NDr:	mov b cs:Man_Keys_1,ah	; Store them

		; Others

		mov ah,0		; None yet

		test b cs:Key_Matrix+kSpace,1; Test for justify
		jz GMK_NS		; No

		or ah,01h		; Set left

GMK_NS:		mov cs:Man_Keys_2,ah	; Store the flags

		ret			; Done

; Wait for an Esc key

Pause:		mov al,CS:Key_Matrix+kEsc; Test for escape
		test al,01H		; End flag
		jz Pause		; Loop

		ret

; Create some random objects, and the rings / key

init_objects:	push ax			; Save
		push cx			;
		push dx			;
		push di			;
		push es			;

		mov cx,No_objects	;

init_objects1:	call Safe_Rand_Pos	; Get an unused cell

		call Rand_32		;

		and al,01Fh		; Get a random object
		add al,0B0h		;

		mov es:[di],al		; Store it

		loop init_objects1	; Loop

		; Now do the rings / key (88 .. 8F

		mov cl,cRings		; 88 .. 8F

init_rings:	call Safe_Rand_Pos	; Get an unused cell

		mov es:[di],cl		; Store it

		inc cl			; Step it

		cmp cl,cKey+1		; All done ?
		jne init_rings		; No, loop

		pop es			; Restore
		pop di			;
		pop dx			;
		pop cx			;
		pop ax			;
		ret			;

Deallocate:	mov bx,o TOS		; Point at top of ss

		mov cl,4		; Convert to paragraphs
		shr bx,cl		;
		inc bx			; add one for luck

		mov cx,ss		; Get stack segment
		add bx,cx		; Add stack segment
		
		mov cx,es		; Get extra segment
		sub bx,cx		; Subtract it

		; bx contains no. of paragraphs from es to the top of ss

		mov ah,Set_Block	; Deallocate surplus RAM
		int dos			;

		mov al,9		; "Internal memory"
		ret			;

; Random number routines (from TP)

Rand_32:	push cx			;
		push bx			;

		call Rand_Inner		; Get next

		xor al,dh		; Scramble it a bit
		xor ah,dl		;

		pop bx			; Restore
		pop cx			;

		ret			; Done

Quick_Rand:	mov ax,cs:Seed		; Giz a word
		ret			;
 
Rand_Inner:	mov ax,CS:Seed		; This is from Turbo Pascal
		mov bx,CS:Seed+2	;
		mov cx,ax		;

		mul cs:Nasty	 	;

		shl cx,1		;
		shl cx,1		;
		shl cx,1		;

		add ch,cl		;
		add dx,cx		;
		add dx,bx		;

		shl bx,1		;
		shl bx,1		;

		add dx,bx		;
		add dh,bl		;

		mov cl,5		;
		shl bx,cl		;

		add dh,bl		;

		add ax,1		;
		adc dx,0		;

		mov CS:Seed,ax		;
		mov CS:Seed+2,dx	;

		ret			;

		; Include the rest of the code

		include MAZE.INC	; Put all the maze garbage here

		include MONSTERS.INC	; Move monster routines
		include MAN.INC		; Man, really a monster
		include WALLFOLL.INC	; Wall following things

		include LINELOOK.INC	; Line drawing, look etc

		include PROJ.INC	; Projectiles, etc

		include IO.INC		; Some IO routines

; Fixed data starts here

sStatus		db "Souls    0  ()()()()()()()() Magic "; Status line
		db 0EEh,16 dup (0EFh), 0EDh		;
		db "  Wounds "				;
		db 0EEh,16 dup (0EFh), 0EDh,0		;

sTitle		dw 8,9			; x,y
		db 98h,"     ",16,17,18,19,20,21,22,23,24,25
		db "     ",98h,0;
		dw 8,10			;
		db 90h,"                    ",90h,0;
		dw 8,11			;
		db 90h,"                    ",90h,0;
		dw 8,12			;
		db 90h,"                    ",90h,0;
		dw 8,13			;
		db 90h,"         ",0AAh,"          ",90h,0;
		dw 8,14			;
		db 94h,"         ",0ABh,"          ",94h,0;

		dw 0			;

sError1		db "Can't get rid of memory",ttr
sError2		db "Can't get enough memory",ttr

Palette		DB 00h,09h,12h,1Bh,24h,2Dh,36h,3Fh
		DB 00h,09h,12h,1Bh,24h,2Dh,36h,3Fh
		DB 0

PowersOfTen	dw 10000,1000,100,10,1	; Powers for decimal print

Kbd_Flags	DB 0			; Some flags

Kbd_Old_Off	DW 0			; Saved vector
Kbd_Old_Seg	DW 0			;

Key_Matrix	DB 128 dup (0)		; Key matrix

Nasty		dw 33797		; Nasty number
Seed		dw 2374,45		; Random number seed

ShowChanged	db ?			;

Man_Keys_1	db ?			; Flags for movement
Man_Keys_2	db ?			;

Soul_Cnt	dw ?			; Status values
Magic_Bar	db ?			;
Wounds_Bar	db ?			;
Rings_Shown	db ?			;

Man_Look	db ?			; Man moved flags

Speed		dw 20000		; Delay value

		; Projectile data

Proj_List	db nProjs*lProj dup (?)	; Some space

Sight_WS_0	db lSight * (nMons/8) dup (?)	;
Sight_WS_E0	equ this byte			;

Sight_WS_1	db lSight * (nMons/8) dup (?)	;
Sight_WS_E1	equ this byte			;

Code		ENDS

		EXTRN tMonochrome:NEAR	; Import translate tables
		EXTRN tColourIBM:NEAR	;
		EXTRN tHallsChars:NEAR	;

		EXTRN Chars:NEAR	;

DATA		SEGMENT BYTE PUBLIC

Small_maze	db (breadth+2)*(height+2) dup (?)

DATA		ENDS

StackSeg	SEGMENT STACK

		DW 100 dup (?)		; Some stack space
TOS		EQU THIS WORD		;

StackSeg	ENDS

		END Start		; Execute address
