; "io !" - greek adjective, a shout used when mounting a cow.

; This file (c) S.Brattel

; These are the routines to drive the keyboard/screen

; These keyboard routines are only used for the front-end

; Any 'keys' waiting ?

Test_Key:	push ax			; Save acc

		; Is there a char waiting ?

		test b cs:Key_Waiting,1 ; Do we have a key ?
		jnz TK_Yes		; Yes, return it

		; Any key-strokes ?

		mov ah,1		; Test if key ready
		int Kbd			;

		jnz TK_Decode		; We have a key-stroke, decode it

		pop ax			; NZ, no keys
		ret			;

TK_Yes:		xor ax,ax		; Z
		inc ax			; NZ

		pop ax			; Return key ready
		ret			;

; We have an IBM key-stroke, we must now decide what to do with it

TK_Decode:	push bx			; Save 'em
		push cx			;
		push dx			; ax is already stacked

		; Get the key-stroke

		mov ah,0		; Read a key (wait)
		int kbd			;

		; Convert IBM 16-bit format to Para-Sys 8-bit format

		cmp al,0		; Special key ?
		jnz TKD_1		; No, got its value

		mov bx,offset Key_Tab-2	; Convert it

TKD_Lp:		inc bx			; Step ptr
		inc bx			;

		cmp b cs:[bx],0		; End of table ?
		jz TKD_1		; Not found

		cmp cs:[bx],ah		; This key ?
		jne TKD_Lp		; No, loop

		mov al,cs:1[bx]		; Convert it

		; al = Key value

TKD_1:		mov b cs:Curr_Key,al	; Store the key

		mov b cs:Key_Waiting,1	; Set the flag

		pop dx			; Restore them
		pop cx			;
		pop bx			;
		pop ax			;

		jmp Test_Key		; Try again

; Look-up the internal key codes

Key_Tab:	DB 48h,cUp		; Up

		DB 4Bh,cLeft		; Left
		DB 73h,cLeft		; Left

		DB 4Dh,cRight		; Right
		DB 74h,cRight		; Right

		DB 50h,cDown		; Down

		DB 47h,0F0h		; Home
		DB 77h,0F4h		; Ctl Home

		DB 49h,0F1h		; PgUp
		DB 84h,0F5h		; Ctl PgUp

		DB 4Fh,0F2h		; End
		DB 75h,0F6h		; Ctl End

		DB 51h,0F3h		; PgDn
		DB 76h,0F7h		; Ctl PgDn

		; Misc

		DB 52h,cInsMode		; Ins
		DB 53h,cRem		; Del

		DB 0Fh,0F8h		; Shift-Tab

		DB 3Bh,cCLS		; Function Keys
		DB 3Ch,cClear_Line	;
		DB 3Dh,cClear_To_End	;
		DB 3Eh,cNull		;
		DB 3Fh,cNull		;
		DB 40h,cLiteral		;
		DB 41h,cAttr0		;
		DB 42h,cAttr1		;
		DB 43h,cAttr2		;
		DB 44h,cAttr3		;

		DB 54h,80h		; Shifted function keys
		DB 55h,81h		;
		DB 56h,82h		;
		DB 57h,83h		;
		DB 58h,84h		;
		DB 59h,85h		;
		DB 5Ah,86h		;
		DB 5Bh,87h		;
		DB 5Ch,88h		;
		DB 5Dh,89h		;

		DB 5Eh,90h		; Alt function keys
		DB 5Fh,91h		;
		DB 60h,92h		;
		DB 61h,93h		;
		DB 62h,94h		;
		DB 63h,95h		;
		DB 64h,96h		;
		DB 65h,97h		;
		DB 66h,98h		;
		DB 67h,99h		;

		DB 68h,0A0h		; Ctrl function keys
		DB 69h,0A1h		;
		DB 6Ah,0A2h		;
		DB 6Bh,0A3h		;
		DB 6Ch,0A4h		;
		DB 6Dh,0A5h		;
		DB 6Eh,0A6h		;
		DB 6Fh,0A7h		;
		DB 70h,0A8h		;
		DB 71h,0A9h		;

		DB 0			; End

; Notice that the cursor can only sensibly be driven by Get_Key since it is
; a flag to the user telling them when the system can accept keystrokes.

Get_Key:	PUSH bx			; Save regs
		PUSH ax			;

		PUSH dx			;
		PUSH cx			;

		MOV al,0		; Fail key code

		CALL vMove_Cur		; Place cursor (if visible)

		CALL Cur_On		; Assert cursor

		; Now wait for a key . . .

GK_Lp:		call Test_Key		; Got one ?
		jz GK_Lp		; No, loop

		; Now we must get the next (buffered) 'key'

		mov al,cs:Curr_Key	; Get the key

		mov b cs:Key_Waiting,0	; Clear the flag

		; Clear the cursor

		CALL Cur_Off		; Negate cur

Get_Key_Fail:	POP cx			; Done
		POP dx			;

		POP bx			;
		POP bx			;

		RET			;

; Drive the cursor around

Cur_On:		PUSH cx			; Save
		PUSH ax			;

		MOV ch,10		; Start line
		MOV cl,12		; End line

		MOV ah,1		; Set size
		INT vdu			; Call it

		POP ax			;
		POP cx			;
		RET			;

Cur_Off:	PUSH cx			; Save
		PUSH ax			;

		MOV ch,20h		; Start line
		MOV cl,ch		; End line

		MOV ah,1		; Set size
		INT vdu			; Call it

		POP ax			;
		POP cx			;
		RET			;

vMove_Cur:	ret			; Done

; Now we must setup a valid display

Init_VDU:	mov b cs:UDC_Flag,0	; Assume no user defined chars

		; Now inspect the video adaptor

		mov ax,0B000h		; Assume monochrome
		mov cs:Screen_Page,ax	;

		mov ax,o tMonochrome	; Assume monochrome translate table
		mov cs:pChar_Trans,ax	; 

		mov b cs:Screen_Mode,7	; Assume monochrome mode

		; Presume the monochrome attr

		mov b cs:Low_Attr,07h	; Lowlight
		mov b cs:High_Attr,0Dh	; Highlight
		mov b cs:Cursor_Attr,17h; Cursor

		; Now find current

		mov ah,15		; Get video mode
		int vdu			;

		mov cs:Old_VDU_Mode,al	; Save mode

		cmp al,7		; Monochrome ?
		je IVDU_ModeSet		; Yes, this is a valid mode

		; Setup for colour

		mov w cs:Screen_Page,0B800h ; Colour segment

		mov ax,o tColourIBM	; Setup translate table
		mov cs:pChar_Trans,ax	;

		; Use the colour attr

		mov b cs:Low_Attr,07h	; Lowlight
		mov b cs:High_Attr,0Dh	; Highlight
		mov b cs:Cursor_Attr,17h; Cursor

		; Setup the mode

		mov al,3		; Else set it to 80x25 colour
		mov cs:Screen_Mode,al	; Store it

		mov ah,0		;
		int vdu			;

		; Get the current page, and hide the cursor

IVDU_ModeSet:	mov ah,15		; Get new mode
		int vdu			;

		mov cs:VDU_Page,bh	; Save page

		; Move the cursor

		mov dx,02051h		; Hide the cursor off screen

		mov ah,2		; Set cur pos
		int vdu			;

		; Check if we can change char set

		mov ax,cs:Screen_Page	; Mono or colour ?
		cmp ax,0B000h		; Mono ?
		je IV_Done		; Yes, can't change chars

		; If it is just colour, we must use silly chars

		mov b cs:UDC_Flag,1	; Lets assume EGA/VGA

		; Now setup the translate table

		mov ax,o tHallsChars	; Using standard(ish) halls chars
		mov cs:pChar_Trans,ax	; 

		; Done, exit

IV_Done:	ret			; Done

; Set screen to front-end mode

Set_VDU_Front:	mov al,cs:Screen_Mode	; Giz the mode
		mov ah,0		;

		int vdu			; Set the mode

		; Remove the cursor

		call Cur_Off		; Turn the cursor off

		; Clear the display

		call CLS		; Clear the display

		ret			; Done, exit

; Set screen to game mode

Set_VDU_Game:	mov al,cs:Screen_Mode	; Giz the mode
		mov ah,0		;

		int vdu			; Set the mode

		; Clear the display

		call CLS		; Clear the display

		test b cs:UDC_Flag,0FFh	; Can we define the characters ?
		jz SVG_End		; No, don't define them

		; Define the chars for EGA/VGA

		push es			; Save it
		push ds			;

		mov ax,DATA		; Point at DATA segment
		mov es,ax		;

		; Define character set 0 chars 0..FF

		mov ax,1110h		; Define chars
		mov bx,0E00h		; 14 bytes per char
		mov cx,100h		; Define the first 32
		mov dx,0		; No offset

		mov bp,o Chars		; Point at them
		int vdu			; Change them

		; Define character set 1 chars 0..FF

		mov ax,1110h		; Define chars
		mov bx,0E01h		; 14 bytes per char
		mov cx,100h		; Define the first 32
		mov dx,0		; No offset

		mov bp,o Chars+(256*14)	; Point at them
		int vdu			; Change them

		mov ax,1110h		; Define chars
		mov bx,0E01h		; 14 bytes per char
		mov cx,80h		; Define the last 128
		mov dx,80h		; No offset

		mov bp,o Chars+(384*14)	; Point at them
		int vdu			; Change them

		; Now activate the 512 char mode using (set 0:1)

		mov ax,1103h		; Activate the char set
		mov bl,04h		; SET 01 00

		int vdu			; Do it

		; Assume EGA and change palette

		mov ax,01002h		; 10,02

		push cs			; Get seg
		pop es			;

		mov dx,o Palette	; Point at data

		int vdu			; Try it

		pop ds			; Restore
		pop es			;

		; Remove the cursor

SVG_End:	call Cur_Off		; Turn the cursor off

		ret			;

; Restore the screen

Close_VDU:	mov al,cs:Old_VDU_Mode	; Get old mode
		mov ah,0		;
		int vdu			;

		ret			; Done


; Here is the keyboard driving code. This is AT specific at the moment, we must
; also drive XT's and Amstrads.

Link_Kbd:	XOR ax,ax		; Clear flags
		MOV CS:Kbd_Flags,al	;

		PUSH es			; Save segs
		PUSH ds			;

		MOV al,09H		; Kbd vector

		MOV ah,035H		; Read vector
		INT dos			;

		MOV CS:Kbd_Old_Off,bx	; Save it
		MOV bx,es		;
		MOV CS:Kbd_Old_Seg,bx	; Save it

		PUSH cs			; Get segment
		POP ds			;

		MOV dx,o Kbd_Interrupt	; Assert it
		MOV al,09H		; Keyboard int vector

		MOV ah,25H		; Set interrupt vector
		INT dos			;

		POP ds			; Restore
		POP es			;

		RET			; Done

Unlink_Kbd:	PUSH ds			; Save ds

		MOV dx,CS:Kbd_Old_Seg	; Get old vector
		MOV ds,dx		;
		MOV dx,CS:Kbd_Old_Off	;

		MOV al,09H		; Kbd int

		MOV ah,25H		; Restore it
		INT dos			;

		POP ds			; Restore

		CLC			; OK
		RET			; Done

Kbd_Interrupt:	STI			; EI

		PUSH ax			; Save regies
		PUSH bx			;
		PUSH cx			;
		PUSH dx			;

		PUSH di			;
		PUSH si			;

		MOV al,0ADH		; Disable keyboard
		CALL SendToKbd		; Ship it

		CLI			; DI
		SUB cx,cx		; Clear count

KI_Lp1:		IN al,Kbd_Status	; Buffer full ?
		TEST al,02H		;
		LOOPNZ KI_Lp1		; Loop

		IN al,Kbd_Data		; Get the character from t'kbd
		STI			; Enable ints again

		CMP al,0FAH		; Is this an ACK ?
		JE KI_Ack		; Yes, set the flag and ignore it.

		CMP al,0FEH		; Is this a resend ?
		JE KI_Resend		; Yes, just ignore it (for now)

		MOV bl,al		; Create ptr to Key Matrix
		AND bx,007Fh		;
		ADD bx,o Key_Matrix	;

		ROL al,1		; Flag to bit 0
		NOT al			; Invert state
		AND al,01h		; 0 = Not pressed, 1 = Pressed

		MOV CS:[bx],al		; Save key state

Kbd_I_Ret:	CLI			; DI

		MOV al,EOI		; End of interrupt
		OUT Int_Ctrl,al		; Tell 8259

		MOV al,0AEH		; Enable kbd
		CALL SendToKbd		; Tell kbd

		CLI			; DI

		POP si			; Restore regs
		POP di			;

		POP dx			;
		POP cx			;
		POP bx			;
		POP ax			;

		IRET			; Done, restore flags

KI_Resend:	
KI_Ack:		OR b CS:Kbd_Flags,01H	; Ack recieved
		JMP SHORT Kbd_I_Ret	; Done

SendToKbd:	PUSH ax			; Talk to Kbd

		CLI			; DI
		SUB cx,cx		; Clear count

STK_Lp:		IN al,Kbd_Status	; Buffer full ?
		TEST al,02H		;
		LOOPNZ STK_Lp		; Loop

		POP ax			; Restore data
		OUT Kbd_Status,al	; Send it

		STI			; EI
		RET			; Done


; Here we have some routines to drive the screen


; Clear the entire window

CLS:		push es			; Save segment

		mov es,cs:Screen_Page	; Point at screen

		mov di,0		; Start of screen

		mov cx,80*25		; Number of chars

		mov ax,0720h		; White space
		rep stosw		; Store word

		pop es			; Done
		ret			;

; Given es:di = ptr to the maze and al = char al on screen if visible

ShowAddr:	push di			; Save

		; Calc offset

		sub di,cs:SA_Off	; Calc relative offset

		; Visible ?

		cmp di,Screen_Width	; In sight ?
		jae SA_F		; No, fail

		push bx			; Save reg

		mov bx,es		; Get Y

		sub bx,cs:SA_Seg	; Calc relative offset

		; Visible ?

		cmp bx,Screen_Height * BM_Y_Stride ; In sight ?
		jae SA_F1			   ; No, fail

		; Save more regs

		push es			; Save regs
		push ax			;

		; Convert to screen segment

		add di,di		; * 4
		add di,di		;

		shr bx,1		; There are 40 paragraphs per line in
		shr bx,1		; maze, and 10 on the screen.

		add bx,cs:Screen_Page	; Segment

		mov es,bx		; Setup Y

		; Now look-up the character

		mov bl,al		; Giz char
		mov bh,0		; 16 - bit

		add bx,bx		; * 4
		add bx,bx		;

		add bx,cs:pChar_Trans	; Add base of char translate table

		; Write to the screen

		mov ax,cs:[bx]		; Get char and attribute
		mov es:[di],ax		; Show it

		inc di			; Step ptr
		inc di			;

		mov ax,cs:2[bx]		; Next bit
		mov es:[di],ax		;

		; Done

		pop ax			; Restore		
		pop es			;

SA_F1:		pop bx			;

SA_F:		pop di			;

		ret			;

; Shows a window onto the maze from (ScreenX,ScreenY) - top left
; This code assumes a text-display format

SW_X:		ret			; Done

Show_Window:	test b cs:ShowChanged,0FFh ; Changed ?
		je SW_X			   ; No, skip

ReShow_Window:	push di			; Save regs
		push si			;

		push dx			;
		push cx			;
		push bx			;
		push ax			;

		push bp			;

		mov dx,cs:ScreenX	; x
		mov bp,cs:ScreenY	; y

		; Calc top lhs values for ShowAddr

		mov cs:SA_Off,dx	; Offset

		mov ax,bp		; Y
		
		push dx			; Save

		mov cx,BM_Y_Stride	; * Y offset
		mul cx			;

		pop dx			;

		add ax,cs:BM_Base	; Start of BM

		mov cs:SA_Seg,ax	; Seg

		; Now show the maze

		mov ax,cs:Screen_Page	; Setup segment
		mov es,ax		;

		mov di,0		; Offset to screen

		mov cl,Screen_Height	; Y Loop
		mov ch,0		;

SW_LpY:		push cx			; Save Y cnt

		cmp bp,BM_Max_Y		; Check for Y out of range
		ja SW_Y_Fail		; In moat

		mov ax,bp		; Get Y

		; Calc Y*40

		mov cx,ax		; Save

		add ax,ax		;
		add ax,ax		; * 4

		add ax,cx		; * 5

		add ax,ax		;
		add ax,ax		;
		add ax,ax		; * 40

		add ax,cs:BM_Base	; Start of BM
		mov ds,ax		; As data seg.

		mov si,dx		; Get ptr to X

		mov cl,Screen_Width	; X Loop
		mov ch,0		;

SW_LpX:		cmp si,BM_Max_X		; Check if X in range
		ja SW_X_Fail		; In moat

		mov al,[si]		; Read char from maze

		mov bx,cs:pChar_Trans	; Get ptr to char translate table

		mov ah,0		; Make 16-bit

		add ax,ax		; Char * 2
		add ax,ax		;      * 4

		add bx,ax		; Index

		mov ax,cs:[bx]		; Get char and attribute
		mov es:[di],ax		; Show it

		inc di			; Step ptr
		inc di			;

		mov ax,cs:2[bx]		; Next bit
		mov es:[di],ax		;

		inc di			; Step ptr
		inc di			;

SW_StepX:	inc si			; Step ptr to maze

		loop SW_LpX		; Loop

SW_DoneX:	inc bp			; Step Y

		pop cx			; Done one line, step Y
		loop SW_LpY		; Loop

		; Negate flag

		mov b cs:ShowChanged,0	; Clear it

		pop bp			; Restore regs.

		pop ax			;
		pop bx			;
		pop cx			;
		pop dx			;

		pop si			;
		pop di			;

		ret			; Done

SW_X_Fail:	mov ax,0FFh*4		; Moat char

		mov bx,cs:pChar_Trans	; Get ptr to char translate table

		add bx,ax		; Index

		mov ax,cs:[bx]		; Get char and attribute
		mov bx,cs:2[bx]		;

		mov es:[di],ax		; Store it
		mov es:2[di],bx		;

		add di,4		; Step ptr

		jmp SW_StepX		; Done

SW_Y_Fail:	mov ax,0FFh*4		; Moat char

		mov bx,cs:pChar_Trans	; Get ptr to char translate table

		add bx,ax		; Index

		mov ax,cs:[bx]		; Get char and attribute
		mov bx,cs:2[bx]		;

		mov cl,Screen_Width	; Whole width of screen
		mov ch,0		;

SW_Y_F_Lp:	mov es:[di],ax		; Store it
		mov es:2[di],bx		;

		add di,4		; Step ptr

		loop SW_Y_F_Lp		; Loop

		jmp SW_DoneX		; Next line

; Show the graphics character set

Show_Chars:	mov al,0		; Char := 0

		mov dh,8		; Y := 8

Lp1:		mov dl,0		; X := 0

Lp2:		call OutIt		; Show it

		inc al			; Step char

		add dl,2		; Step X

		cmp dl,32		; Done ?
		jne Lp2			; No, loop

		inc dh			; Step Y

		cmp dh,16+8		; Done ?
		jne Lp1			; No, loop

		mov	di,o sMen_Top	; Put up men
		mov	dl,40		; X
		mov	dh,6		; Y
		call	print_str_code	;

		mov	di,o sMen_Bottom; Put up men
		mov	dl,40		; X
		mov	dh,7		; Y
		call	print_str_code	;

		ret			; Done

; Given dl = Sx and dh = Sy write char al to screen

OutIt:		cmp dl,Screen_Width	; X in range ?
		jae OutIt_Fail		; No, done

		cmp dh,Screen_Height+1	; Y in range ?
		jae OutIt_Fail		; No, done

		push di			; Save ptr
		push bx			; Save ptr

		push ax			; Save char

		mov bl,dl		; X
		mov bh,0		;

		add bx,bx		; 2.X

		mov al,Screen_Width	; Convert to screen address
		mov ah,0		;

		add ax,ax		; *2

		mul dh			; 2.Width.Y

		add bx,ax		; 2.Width.Y + 2.X

		add bx,bx		; 2(2.Width.Y + 2.X)

		mov di,bx		; Use another ptr

		pop ax			; Get char
		push ax			;

		mov bx,cs:pChar_Trans	; Get ptr to char translate table

		mov ah,0		; Make 16-bit
		add ax,ax		; Char * 2
		add ax,ax		; * 4

		add bx,ax		; Index

		mov ax,cs:[bx]		; Get char and attribute
		mov bx,cs:2[bx]		; Next char

		push ds			; Save seg

		mov ds,cs:Screen_Page	; Segment

		mov [di],ax		; Write char
		mov 2[di],bx		; next char

		pop ds			; Restore

		pop ax			;

		pop bx			;
		pop di			;

OutIt_Fail:	ret			; Done

; Write decimal ax = no, dx = x,y

Show_No:	PUSH ax			; Save regs
		PUSH bx			;
		PUSH cx			;

		MOV cx,ax		; Shuffle regs

		MOV bx,offset PowersOfTen ; Digit values table

		MOV ah,' '		; To be displayed for 0

WD_Lp:		CALL WD_CalcDigit	; Calc digit

		CMP al,0		; Zero ?
		JNZ WD_1		; No

		MOV al,ah		; Print ' ' or '0'

		CALL OutItText		;
		inc dl			; Step X

		JMP WD_Next		; Do next digit

WD_1:		MOV ah,'0'		; Use 0 in future
		
		OR al,ah		; Make char

		CALL OutItText		;
		inc dl			;

WD_Next:	ADD bx,2		; Next power of ten

		CMP w CS:[bx],1		; Last power of ten ?
		JNZ WD_Lp		; No, continue

		CALL WD_CalcDigit	; Calc digit

		OR al,'0'		; Make char

		CALL OutItText		;
		inc dl			;

		POP cx			;
		POP bx			;
		POP ax			;

		RET			; Done
		
WD_CalcDigit:	MOV al,-1		; Initial digit
	
WDCD_Lp:	INC al			; Inc digit

		SUB cx,CS:[bx]		; Low word
		JNC WDCD_Lp		; Try again

		ADD cx,CS:[bx]		; Step back

		RET			; Done						

; Drive a bar-graph, dx = x,y al = 0 .. 64

Draw_Bar:	push cx			; Save it
		push ax			;

		mov ah,al		; Get 0 .. 64

		mov cx,16		; N segments

		mov al,0ECh		; Full segment

DB_Lp:		sub ah,4		; Out of this segment ?
		jb DB_1			; No, draw end

		call OutItText		; Write it
		inc dl			; Step X

		loop DB_Lp		; Loop

		pop ax			; Restore
		pop cx			;
		ret			; Done

DB_1:		add al,ah		; Calc the next char

DB_Lp2:		call OutItText		; Show it
		inc dl			; Step X

		mov al,0EFh		; Empty

		loop DB_Lp2		; Loop

		pop ax			; Restore
		pop cx			;

		ret			; Done

; Write a string to the display dl = x, dh = y, di = pStrZ

OutTextStr:	mov al,cs:[di]		; Giz a char
		cmp al,0		; End
		jz OutIt_Fail		; Done

		call OutItText		; Write the char

		inc dl			; Step X

		inc di			; Step ptr

		jmp s OutTextStr	; Loop		

; Given dl = Sx and dh = Sy write text char al to screen

OutItText:	cmp dl,Screen_Width*2	; X in range ?
		jae OutItT_Fail		; No, done

		cmp dh,1		; Y in range ?
		jae OutItT_Fail		; No, done

		push di			; Save ptr
		push dx			;
		push bx			; Save ptr

		push ax			; Save char

		mov bl,dl		; X
		mov bh,0		;

		add bx,bx		; 2.X

		mov al,Screen_Width*2	; Convert to screen address
		mov ah,0		;

		add ax,ax		; *2

		add dh,Screen_Height	; Move to bottom 3 lines

		mul dh			; 2.Width.Y

		add bx,ax		; 2.Width.Y + 2.X

		mov di,bx		; Use another ptr

		pop ax			; Get char
		push ax			;

		mov bx,cs:pChar_Trans	; Get ptr to char translate table

		mov ah,0		; Make 16-bit
		add ax,ax		; Char * 2
		add ax,ax		; * 4

		add bx,ax		; Index

		mov ax,cs:[bx]		; Get char and attribute

		push ds			; Save seg

		mov ds,cs:Screen_Page	; Segment

		mov [di],ax		; Write char

		pop ds			; Restore

		pop ax			;

		pop bx			;
		pop dx			;
		pop di			;

OutItT_Fail:	ret			; Done

Old_VDU_Mode	db ?			; Original VDU mode
VDU_Page	db ?			; Display page

Screen_Page	dw ?			; Display segment
Screen_Mode	db ?			; Display mode
UDC_Flag	db ?			; UDC possible flag

pChar_Trans	dw ?			; Ptr to translate table

ScreenX		dw ?			; Window position
ScreenY		dw ?			;

SA_Off		dw ?			;
SA_Seg		dw ?			;

Low_Attr	db ?			; Current attrs
High_Attr	db ?			;
Cursor_Attr	db ?			;

Curr_Key	db 0			; Current key code
Key_Waiting	db 0			; Flag
